==> slingpy-0.1.3-py3-none-any.whl: slingpy/utils/argument_dictionary.py <==
"""
ADAPTED FROM: scikit-learn

Author: Gael Varoquaux <gael.varoquaux@normalesup.org>
Original License:

BSD 3-Clause License

Copyright (c) 2007-2021 The scikit-learn developers.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
  list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.

* Neither the name of the copyright holder nor the names of its
  contributors may be used to endorse or promote products derived from
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Copyright (C) 2021  Patrick Schwab, Arash Mehrjou, GlaxoSmithKline plc; Andrew Jesson, University of Oxford; Ashkan Soleymani, MIT

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
documentation files (the "Software"), to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions
 of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""
import six
import struct
import inspect
import hashlib
from collections import defaultdict
from collections.abc import Iterable
from typing import List, AnyStr, Dict


class ArgumentDictionary(object):
    @classmethod
    def _get_param_names(cls) -> List[AnyStr]:
        """ Get argument names for this class. """
        # fetch the constructor or the original constructor before
        # deprecation wrapping if any
        init = getattr(cls.__init__, 'deprecated_original', cls.__init__)
        if init is object.__init__:
            # No explicit constructor to introspect
            return []

        # introspect the constructor arguments to find the model parameters
        # to represent
        init_signature = inspect.signature(init)
        # Consider the constructor parameters excluding 'self'
        parameters = [p for p in init_signature.parameters.values()
                      if p.name != 'self' and p.kind != p.VAR_KEYWORD]
        for p in parameters:
            if p.kind == p.VAR_POSITIONAL:
                raise RuntimeError("scikit-learn estimators should always "
                                   "specify their parameters in the signature"
                                   " of their __init__ (no varargs)."
                                   " %s with constructor %s doesn't "
                                   " follow this convention."
                                   % (cls, init_signature))
        # Extract and sort argument names excluding 'self'
        return sorted([p.name for p in parameters])

    def _get_bytes(self, value) -> bytes:
        if isinstance(value, six.string_types):
            return bytes(value, 'utf-8')
        elif isinstance(value, float):
            return bytearray(struct.pack("!f", value))
        elif isinstance(value, int):
            return value.to_bytes(value.bit_length(), byteorder='big')
        elif isinstance(value, bool):
            return bytes(value)
        elif value is None:
            the_hash = 0
            return the_hash.to_bytes(the_hash.bit_length(), byteorder='big')
        elif hasattr(value, "__hash__") and value.__hash__ is not None:
            the_hash = value.__hash__()
            the_hash = the_hash.to_bytes(the_hash.bit_length(), byteorder='big')
            return the_hash
        elif isinstance(value, Iterable):
            the_hash = b"".join([self._get_bytes(it) for it in value])
            return the_hash
        else:
            raise NotImplementedError(f"{value} is not hashable (does not implement __hash__).")

    def get_params_hash(self, ignored_params=None) -> AnyStr:
        if ignored_params is None:
            ignored_params = []
        ignored_params = set(ignored_params)
        m = hashlib.sha256()
        params, val = self.get_params(), 0
        for key in sorted(list(set(params.keys()) - ignored_params)):
            value = self._get_bytes(params[key])
            m.update(value)
        return m.hexdigest()

    def get_params(self, deep=True) -> Dict:
        """
        Get parameters for this estimator.

        Args:
            deep: bool, default=True
             If True, will return the parameters for this estimator and
             contained subobjects that are estimators.

        Returns:
            Parameter names mapped to their values.

        """
        out = dict()
        for key in self._get_param_names():
            value = getattr(self, key)
            if deep and hasattr(value, 'get_params'):
                deep_items = value.get_params().items()
                out.update((key + '__' + k, val) for k, val in deep_items)
            out[key] = value
        return out

    def set_params(self, **params) -> "ArgumentDictionary":
        """
        Set the parameters of this estimator.

        Args:
            **params: Instance arguments.

        Returns:
            The instance.
        """
        if not params:
            # Simple optimization to gain speed (inspect is slow)
            return self
        valid_params = self.get_params(deep=True)

        nested_params = defaultdict(dict)  # grouped by prefix
        for key, value in params.items():
            key, delim, sub_key = key.partition('__')
            if key not in valid_params:
                raise ValueError('Invalid parameter %s for instance %s. '
                                 'Check the list of available parameters '
                                 'with `instance.get_params().keys()`.' %
                                 (key, self))

            if delim:
                nested_params[key][sub_key] = value
            else:
                setattr(self, key, value)
                valid_params[key] = value

        for key, sub_params in nested_params.items():
            valid_params[key].set_params(**sub_params)
        return self

==> slingpy-0.2.12-py3-none-any.whl: slingpy/utils/argument_dictionary.py <==
"""
ADAPTED FROM: scikit-learn

Author: Gael Varoquaux <gael.varoquaux@normalesup.org>
Original License:

BSD 3-Clause License

Copyright (c) 2007-2021 The scikit-learn developers.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
  list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.

* Neither the name of the copyright holder nor the names of its
  contributors may be used to endorse or promote products derived from
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Copyright (C) 2021  Patrick Schwab, Arash Mehrjou, GlaxoSmithKline plc; Andrew Jesson, University of Oxford; Ashkan Soleymani, MIT

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
documentation files (the "Software"), to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions
 of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""
import hashlib
import inspect
import struct
from collections import defaultdict
from collections.abc import Iterable
from typing import AnyStr, Dict, List


class ArgumentDictionary:
    @classmethod
    def _get_param_names(cls) -> List[AnyStr]:
        """Get argument names for this class."""
        # fetch the constructor or the original constructor before
        # deprecation wrapping if any
        init = getattr(cls.__init__, "deprecated_original", cls.__init__)
        if init is object.__init__:
            # No explicit constructor to introspect
            return []

        # introspect the constructor arguments to find the model parameters
        # to represent
        init_signature = inspect.signature(init)
        # Consider the constructor parameters excluding 'self'
        parameters = [p for p in init_signature.parameters.values() if p.name != "self" and p.kind != p.VAR_KEYWORD]
        for p in parameters:
            if p.kind == p.VAR_POSITIONAL:
                raise RuntimeError(
                    "scikit-learn estimators should always "
                    "specify their parameters in the signature"
                    " of their __init__ (no varargs)."
                    " %s with constructor %s doesn't "
                    " follow this convention." % (cls, init_signature)
                )
        # Extract and sort argument names excluding 'self'
        return sorted([p.name for p in parameters])

    def _get_bytes(self, value) -> bytes:
        if isinstance(value, str):
            return bytes(value, "utf-8")
        elif isinstance(value, float):
            return bytearray(struct.pack("!f", value))
        elif isinstance(value, int):
            return value.to_bytes(value.bit_length(), byteorder="big")
        elif isinstance(value, bool):
            return bytes(value)
        elif value is None:
            the_hash = 0
            return the_hash.to_bytes(the_hash.bit_length(), byteorder="big")
        elif hasattr(value, "__hash__") and value.__hash__ is not None:
            the_hash = value.__hash__()
            the_hash = the_hash.to_bytes(the_hash.bit_length(), byteorder="big", signed=the_hash < 0)
            return the_hash
        elif isinstance(value, Iterable):
            the_hash = b"".join([self._get_bytes(it) for it in value])
            return the_hash
        else:
            raise NotImplementedError(f"{value} is not hashable (does not implement __hash__).")

    def get_params_hash(self, ignored_params=None) -> AnyStr:
        if ignored_params is None:
            ignored_params = []
        ignored_params = set(ignored_params)
        m = hashlib.sha256()
        params, val = self.get_params(), 0
        for key in sorted(list(set(params.keys()) - ignored_params)):
            value = self._get_bytes(params[key])
            m.update(value)
        return m.hexdigest()

    def get_params(self, deep=True) -> Dict:
        """
        Get parameters for this estimator.

        Args:
            deep: bool, default=True
             If True, will return the parameters for this estimator and
             contained subobjects that are estimators.

        Returns:
            Parameter names mapped to their values.

        """
        out = dict()
        for key in self._get_param_names():
            value = getattr(self, key)
            if deep and hasattr(value, "get_params"):
                deep_items = value.get_params().items()
                out.update((key + "__" + k, val) for k, val in deep_items)
            out[key] = value
        return out

    def set_params(self, **params) -> "ArgumentDictionary":
        """
        Set the parameters of this estimator.

        Args:
            **params: Instance arguments.

        Returns:
            The instance.
        """
        if not params:
            # Simple optimization to gain speed (inspect is slow)
            return self
        valid_params = self.get_params(deep=True)

        nested_params = defaultdict(dict)  # grouped by prefix
        for key, value in params.items():
            key, delim, sub_key = key.partition("__")
            if key not in valid_params:
                raise ValueError(
                    "Invalid parameter %s for instance %s. "
                    "Check the list of available parameters "
                    "with `instance.get_params().keys()`." % (key, self)
                )

            if delim:
                nested_params[key][sub_key] = value
            else:
                setattr(self, key, value)
                valid_params[key] = value

        for key, sub_params in nested_params.items():
            valid_params[key].set_params(**sub_params)
        return self
